from horsetalk import RacePerformance as HorsetalkRacePerformance

from analysis.rating import FormRating, TimeRating


class RacePerformance(HorsetalkRacePerformance):
    def __init__(
        self,
        *args,
        form_rating: FormRating | None = None,
        time_rating: TimeRating | None = None,
        **kwargs,
    ):
        self.form_rating = form_rating
        self.time_rating = time_rating
        super().__init__(*args, **kwargs)
